/*****************************************************************************

	[bookshop.cpp] BookShop class implementation
	By Jeff Leung

	All BookShop member functions return a variable of type 
	'BookShop::BookShop::return_value' unless otherwise specified 
	(constructors and destructors return nothing as always).

*****************************************************************************/
#include "bookshop.h"
#include "misc.h"

//_[BookShop::BookShop()]_____________________________________________________
//
// DESCRIPTION:
//  Default constructor; doesn't do anything.
//
// PARAMETERS:
//  None.
//
BookShop::BookShop()
{
}

//_[BookShop::~BookShop()]____________________________________________________
//
// DESCRIPTION:
//  Frees all memory used by client and book objects.
//
BookShop::~BookShop()
{
	//_[free memory used by all book objects]_________________________________
	if (aBooks.size() > 0)
	{
		vector<Book*>::iterator pBook;

		for (pBook = aBooks.begin(); pBook != aBooks.end(); pBook++)
			delete *pBook;
	}

	//_[free memory used by all client objects]_______________________________
	if (aClients.size() > 0)
	{
		vector<Client*>::iterator pClient;

		for (pClient = aClients.begin(); pClient != aClients.end(); pClient++)
			delete *pClient;
	}
}

//_[BookShop::List()]_________________________________________________________
//
// DESCRIPTION:
//
// PARAMETERS:
//
BookShop::return_value BookShop::List(filter_type iType, unsigned int iStart)
{
	string sID;
	bool fCount;
	unsigned int i = 0, iPage = 0;
	string sLoop = "y";

	//_[list books]___________________________________________________________
	if (iType & AllBooks)
	{
		vector<Book*>::iterator pBookI;

		//-[if there are no books, return an error]---------------------------
		if (!aBooks.size())
			return NoBooks;

		//-[make sure we start at a valid index]------------------------------
		if (iStart >= aBooks.size())
			iStart = 0;

		sID = aBooks[0]->getIDProp();		// get id property used by books

		//-[output header]----------------------------------------------------
		cout << "Book   | " << sID << endl;
		cout << " - - - + - - " << endl;

		//-[iterate through all books]----------------------------------------
		for (pBookI = aBooks.begin(); sLoop[0] != 'n' && pBookI != aBooks.end(); pBookI++)
		{
			fCount = false;

			//-[decide, based on filter, to count this book or not]-----------
			if (((iType & InBooks) == InBooks) &&
				!(*pBookI)->isBorrowed())
				fCount = true;
			if (((iType & NotDueBooks) == NotDueBooks) &&
				((*pBookI)->isBorrowed() && !(*pBookI)->isDue()))
				fCount = true;
			if (((iType & DueBooks) == DueBooks) &&
				(*pBookI)->isDue())
				fCount = true;
				
			if (fCount)
			{
				i++;
				if ((i-1) >= iStart)
				{
					cout << setw(6) << (i-1) << " | " << (*pBookI)->getValue(sID) << endl;

					//-[decide if we have listed enough for a page]-----------
					iPage++;
					if (iPage >= 10)
					{
						cout << "Continue listing (y/n)? ";
						getline(cin, sLoop);
						sLoop[0] = toupper(sLoop[0]);
						iPage = 0;
					}
				}
			}
		}
	}
	//_[list clients]_________________________________________________________
	else if (iType & AllClients)
	{
		vector<Client*>::iterator pClientI;

		//-[if there are no clients, return an error]-------------------------
		if (!aClients.size())
			return NoBooks;

		//-[make sure we start at a valid index]------------------------------
		if (iStart >= aClients.size())
			iStart = 0;

		sID = aClients[0]->getIDProp();		// get id property used by clients

		//-[output header]----------------------------------------------------
		cout << "Client | " << sID << endl;
		cout << " - - - + - - -" << endl;

		//-[iterate through all clients]--------------------------------------
		for (pClientI = aClients.begin(); sLoop[0] != 'n' && pClientI != aClients.end(); pClientI++)
		{
			fCount = false;

			//-[decide, based on filter, to count this book or not]-----------
			if (((iType & EmptyClients) == EmptyClients) &&
				!(*pClientI)->isBorrowing())
				fCount = true;
			if (((iType & GoodClients) == GoodClients) &&
				((*pClientI)->isBorrowing() && !(*pClientI)->hasDue()))
				fCount = true;
			if (((iType & BadClients) == BadClients) &&
				(*pClientI)->hasDue())
				fCount = true;
				
			if (fCount)
			{
				i++;
				if ((i-1) >= iStart)
				{
					cout << setw(6) << (i-1) << " | " << (*pClientI)->getValue(sID) << endl;

					//-[decide if we have listed enough for a page]-----------
					iPage++;
					if (iPage >= 10)
					{
						cout << "Continue listing (y/n)? ";
						getline(cin, sLoop);
						sLoop[0] = toupper(sLoop[0]);
					}
				}
			}
		}
	}
	return Okay;
}

//_[BookShop::Tally()]________________________________________________________
//
// DESCRIPTION:
//  Returns a tally based on the filter passed in.
//
// PARAMETERS:
//  iType ................. filter containing wanted totals
//
BookShop::return_value BookShop::Tally(filter_type iType)
{
	unsigned int iTotal;

	//_[count books]__________________________________________________________
	if ((iType & AllBooks) == AllBooks)
	{
		cout << "Total Books: " << aBooks.size() << endl;
	}

	//_[count books borrowed]_________________________________________________
	if ((iType & OutBooks) == OutBooks)
	{
		vector<Book*>::iterator pBookI;
		iTotal = 0;

		for (pBookI = aBooks.begin(); pBookI != aBooks.end(); pBookI++)
		{
			if ((*pBookI)->isBorrowed())
				iTotal++;
		}

		cout << "Books Borrowed: " << iTotal << endl;
	}

	//_[count books borrowed and not due]_____________________________________
	if ((iType & NotDueBooks) == NotDueBooks)
	{
		vector<Book*>::iterator pBookI;
		iTotal = 0;

		for (pBookI = aBooks.begin(); pBookI != aBooks.end(); pBookI++)
		{
			if ((*pBookI)->isBorrowed() && !(*pBookI)->isDue())
				iTotal++;
		}

		cout << "Books Borrowed, but not Due: " << iTotal << endl;
	}

	//_[count books borrowed and due]_________________________________________
	if ((iType & DueBooks) == DueBooks)
	{
		vector<Book*>::iterator pBookI;
		iTotal = 0;

		for (pBookI = aBooks.begin(); pBookI != aBooks.end(); pBookI++)
		{
			if ((*pBookI)->isDue())
				iTotal++;
		}

		cout << "Books Borrowed and Due: " << iTotal << endl;
	}

	//_[count books not borrowed (in)]________________________________________
	if ((iType & InBooks) == InBooks)
	{
		vector<Book*>::iterator pBookI;
		iTotal = 0;

		for (pBookI = aBooks.begin(); pBookI != aBooks.end(); pBookI++)
		{
			if (!(*pBookI)->isBorrowed())
				iTotal++;
		}

		cout << "Books Not Borrowed: " << iTotal << endl;
	}

	//_[count clients]________________________________________________________
	if ((iType & AllClients) == AllClients)
	{
		cout << "Total Clients: " << aClients.size() << endl;
	}

	//_[count clients borrowing]______________________________________________
	if ((iType & BookClients) == BookClients)
	{
		vector<Client*>::iterator pClientI;
		iTotal = 0;

		for (pClientI = aClients.begin(); pClientI != aClients.end(); pClientI++)
		{
			if ((*pClientI)->isBorrowing())
				iTotal++;
		}

		cout << "Clients Borrowing: " << iTotal << endl;
	}

	//_[count clients borrowing with books due]_______________________________
	if ((iType & BadClients) == BadClients)
	{
		vector<Client*>::iterator pClientI;
		iTotal = 0;

		for (pClientI = aClients.begin(); pClientI != aClients.end(); pClientI++)
		{
			if ((*pClientI)->hasDue())
				iTotal++;
		}

		cout << "Clients Borrowing with Books Due: " << iTotal << endl;
	}

	//_[count clients borrowing with no books due]____________________________
	if ((iType & GoodClients) == GoodClients)
	{
		vector<Client*>::iterator pClientI;
		iTotal = 0;

		for (pClientI = aClients.begin(); pClientI != aClients.end(); pClientI++)
		{
			if ((*pClientI)->isBorrowing() && !(*pClientI)->hasDue())
				iTotal++;
		}

		cout << "Clients Borrowing with No Books Due: " << iTotal << endl;
	}

	//_[count clients not borrowing]__________________________________________
	if ((iType & EmptyClients) == EmptyClients)
	{
		vector<Client*>::iterator pClientI;
		iTotal = 0;

		for (pClientI = aClients.begin(); pClientI != aClients.end(); pClientI++)
		{
			if (!(*pClientI)->isBorrowing())
				iTotal++;
		}

		cout << "Clients Not Borrowing: " << iTotal << endl;
	}

	return Okay;
}

//_[BookShop::NewBook()]______________________________________________________
//
// DESCRIPTION:
//  Creates a new book then gets user input on all of their properties.
//
// PARAMETERS:
//  None.
//
BookShop::return_value BookShop::NewBook(void)
{
	return_value iRet;

	//_[allocate new book]____________________________________________________
	Book *pBook = new Book;

	//_[add new client to vector]_____________________________________________
	aBooks.push_back(pBook);

	//_[use generic 'NewItem()' function which works for books & clients]_____
	iRet = NewItem(reinterpret_cast<Item*>(pBook));

	if (iRet == Okay)
		cout << "Book created successfully" << endl;

	return iRet;
}

//_[BookShop::NewClient()]____________________________________________________
//
// DESCRIPTION:
//  Creates a new client then gets user input on all of their properties.
//
// PARAMETERS:
//  None.
//
BookShop::return_value BookShop::NewClient(void)
{
	return_value iRet;

	//_[allocate new client]__________________________________________________
	Client *pClient = new Client;

	//_[add new client to vector]_____________________________________________
	aClients.push_back(pClient);

	//_[use generic 'NewItem()' function which works for books & clients]_____
	iRet = NewItem(reinterpret_cast<Item*>(pClient));

	if (iRet == Okay)
		cout << "Client created successfully" << endl;

	return iRet;
}

//_[BookShop::DeleteBook()]___________________________________________________
//
// DESCRIPTION:
//  Deletes the specified book based on the value of their id property.
//
// PARAMETERS:
//  sBook ................. id of book to delete
//
BookShop::return_value BookShop::DeleteBook(string &sBook)
{
	vector<Book*>::iterator pBook;		// iterator for vector of books
	string sName;						// string to store name of id property

	//_[if there are no books]________________________________________________
	if (!aBooks.size())
		return NoBooks;

	//_[get name of id property for books]____________________________________
	sName = aBooks[0]->getIDProp();

	//_[iterate through all books]____________________________________________
	for (pBook = aBooks.begin(); pBook != aBooks.end(); pBook++)
	{
		//-[if book matches specified id]-------------------------------------
		if ((*pBook)->getValue(sName) == sBook)
		{
			delete *pBook;
			aBooks.erase(pBook, pBook+1);

			cout << "Book '" << sBook << "' deleted." << endl;

			return Okay;
		}
	}

	//_[if book specified was not found]______________________________________
	return BookNoExist;
}

//_[BookShop::DeleteClient()]_________________________________________________
//
// DESCRIPTION:
//  Deletes the specified client based on the value of their id property.
//
// PARAMETERS:
//  sClient ............... id of client to delete
//
BookShop::return_value BookShop::DeleteClient(string &sClient)
{
	vector<Client*>::iterator pClient;	// iterator for vector of clients
	string sName;						// string to store name of id property

	//_[if there are no clients]______________________________________________
	if (!aClients.size())
		return NoClients;

	//_[get name of id property for clients]__________________________________
	sName = aClients[0]->getIDProp();

	//_[iterate through all clients]__________________________________________
	for (pClient = aClients.begin(); pClient != aClients.end(); pClient++)
	{
		//-[if client matches specified id]-----------------------------------
		if ((*pClient)->getValue(sName) == sClient)
		{
			delete *pClient;						// free memory used
			aClients.erase(pClient, pClient+1);		// remove node from vector

			cout << "Client '" << sClient << "' deleted." << endl;

			return Okay;							// return successful
		}
	}

	//_[if client specified was not found]____________________________________
	return ClientNoExist;
}

//_[BookShop::Borrow()]_______________________________________________________
//
// DESCRIPTION:
//  Borrows a book by a client.
//
// PARAMETERS:
//  sBook ................. book to borrow
//  sClient ............... client borrowing book
//
BookShop::return_value BookShop::Borrow(string &sBook, string &sClient)
{
	Book	*pBook		= FindBook(sBook);		// pointer to book borrowed
	Client	*pClient	= FindClient(sClient);	// pointer to borrower
	string	sDate;								// date entered by user
	bool	fGoodDate	= false;				// is date entered good?

	//_[make sure we can borrow this book]____________________________________
	if (pBook == NULL)
		return BookNoExist;

	if (pClient == NULL)
		return ClientNoExist;

	if (pBook->isBorrowed())
		return BookOut;

	//_[get input on due date]________________________________________________
	cout << "Today is " << getTodaysDate() << "\"" << endl;
	do
	{
		cout << "Due Date (DD MMM YYYY): ";
		getline(cin, sDate);
		fGoodDate = isDateValid(sDate);
		if (!fGoodDate)
		{
			cout << "Incorrect format.  Use \"DD MMM YYYY\" like \"01 JAN 2001\"" << endl;
		}
	} while (!fGoodDate);

	//_[let the client object handle the rest of the borrowing]_______________
	if (pClient->Borrow(reinterpret_cast<Item*>(pBook), sDate) == false)
	{
		cout << "Could not borrow book." << endl;
		return Error;
	}

	//_[return successful]____________________________________________________
	cout << "Book successfully borrowed, due back on " << sDate << endl;
	return Okay;
}

//_[BookShop::Return()]_______________________________________________________
//
// DESCRIPTION:
//  Returns a book borrowed by a client.
//
// PARAMETERS:
//  sBook ................. book being returned
//  sClient ............... client returning the book
//
BookShop::return_value BookShop::Return(string &sBook, string &sClient)
{
	Book	*pBook		= FindBook(sBook);		// pointer to book returned
	Client	*pClient	= FindClient(sClient);	// pointer to returner

	//_[make sure the book can be returned]___________________________________
	if (pBook == NULL)
		return BookNoExist;

	if (pClient == NULL)
		return ClientNoExist;

	//_[let client take care of the returning process]________________________
	if (!pClient->Return(pBook))
	{
		cout << "Couldn't return book." << endl;
		return Error;
	}

	//_[return successful]____________________________________________________
	cout << "Book successfully returned." << endl;
	return Okay;
}

//_[BookShop::LoadXML()]______________________________________________________
//
// DESCRIPTION:
//  Loads the data for books and clients from the specified XML file.
//
// PARAMETERS:
//  sFilename ............. name of XML file to load
//
BookShop::return_value BookShop::LoadXML(string &sFilename)
{
	fstream oFile;					// file stream object
	unsigned long iSize;			// size of file
	unsigned int	iBooks = 0,		// number of book objects loaded
					iClients = 0;	// number of client objects loaded
	char 	*sBuf = NULL,			// buffer to store file data
			*pt,					// pointer to character in file buffer
			*wp,					// general use write-copy pointer
			*pC,					// general use character pointer
			*pEnd;					// pointer to position past end of buffer
	char sTag[32], sValue[256];		// general use character strings
	Item *pItem;					// pointer to current new item
	string sAName, sAValue;			// end-result attribute name and value
	Book *pBook;					// general use pointer to book object
	Client *pClient;				// general use pointer to client object
	vector<Book*>::iterator pBookI;	// book vector iterator
	vector<Client*>::iterator pClientI;	// client vector iterator

	//_[open specified file]__________________________________________________
	oFile.open(sFilename.c_str(), ios::in);

	//_[check to see if file was opened]______________________________________
	if (oFile.fail())
		return FileNotFound;

	//_[read whole file into single buffer]___________________________________
	oFile.seekg(0, ios::end);		// seek to end of file
	iSize = oFile.tellg();			// get position at end of file (for size)
	sBuf = new char [iSize];		// allocate buffer to store file data
	pEnd = sBuf + iSize;			// get pointer position past end of buffer
	oFile.seekg(0, ios::beg);		// seek to beginning of file
	oFile.read(sBuf, iSize);		// read file data into buffer
	oFile.close();					// close file

	//_[upper-case all non-literal data]______________________________________
	pt = sBuf;
	while (*pt)
	{
		//-[skip literal data]------------------------------------------------
		if (*pt == '"')
		{
			pt++;
			while (*pt && *pt != '"')
				pt++;
		}
		else
		{
			*pt = toupper(*pt);
		}
		pt++;
	}

	//_[find tags]____________________________________________________________
	pt = sBuf;
	while (*pt)
	{
		//-[found the beginning of tag]---------------------------------------
		if (*pt == '<')
		{
			pt++;

			//-[re-continue loop if empty tag]--------------------------------
			if (*pt == '>')
			{
				pt++;
				continue;
			}

			//-[get past initial whitespace if any]---------------------------
			while (!isalpha(*pt) && *pt != '>' && *pt != '/')
				pt++;

			//-[re-continue if found to be empty]-----------------------------
			if (!isalpha(*pt))
			{
				pt++;
				continue;
			}

			//-[copy in tag name]---------------------------------------------
			wp = sTag;
			while (isalnum(*pt))
			{
				*wp = *pt;
				wp++;
				pt++;
			}
			*wp = '\0';

			//-[determine object to create]-----------------------------------
			if (!strcmp(sTag, "BOOK"))
			{
				pBook = new Book;				// create new book object
				aBooks.push_back(pBook);		// add book object to vector
				
				//-[get generic Item pointer to new book]---------------------
				pItem = reinterpret_cast<Item*>(pBook);

				iBooks++;						// increment number of books
			}
			else if (!strcmp(sTag, "CLIENT"))
			{
				pClient = new Client;			// create new client object
				aClients.push_back(pClient);	// add client object to vector

				//-[get generic Item pointer to new client]-------------------
				pItem = reinterpret_cast<Item*>(pClient);

				iClients++;						// increment number of clients
			}
			else
			{
				//-[skip past unknown tag]------------------------------------
				while (*pt && *pt != '>')
					pt++;
				pt++;
				continue;
			}

			//-[parse attributes]---------------------------------------------
			while (*pt != '>')
			{
				//-[get past pre-white-space]---------------------------------
				while (!isalpha(*pt) && *pt != '>' && *pt != '/')
					pt++;

				if (*pt == '/')
				{
					pt++;
					continue;
				}
				if (*pt == '>')
				{
					pt++;
					break;
				}

				//-[copy in attribute name]-----------------------------------
				wp = sTag;
				while (isalnum(*pt))
				{
					*wp = *pt;
					wp++;
					pt++;
				}
				*wp = '\0';
				sAName = sTag;

				//-[get to value]---------------------------------------------
				while (*pt != '"')
					pt++;

				//-[copy in value]--------------------------------------------
				wp = sValue;
				pt++;
				while (*pt != '"')
				{
					*wp = *pt;
					wp++;
					pt++;
				}
				*wp = '\0';
				pt++;

				//-[unescape characters]--------------------------------------
				pC = sValue;
				sAValue = "";
				while (*pC)
				{
					if (*pC == '&')
					{
						pC+=2;
						wp = sTag;
						while (*pC != ';')
						{
							*wp = *pC;
							wp++;
							pC++;
						}
						*wp = '\0';
						sAValue += (char)strtol(sTag, NULL, 10);
					}
					else
					{
						sAValue += *pC;
					}
					pC++;
				}

				//-[save property to current item]----------------------------
				pItem->setProp(sAName, sAValue);
			}	
		}
		pt++;
	}

	//_[delete file data buffer]______________________________________________
	delete sBuf;

	//_[restore book/client link pointers]____________________________________
	if (aBooks.size())
	{
		sAName = "BORROWER";

		for (pBookI = aBooks.begin(); pBookI != aBooks.end(); pBookI++)
		{
			sAValue = (*pBookI)->getValue(sAName);
			if(sAValue != "")
			{
				pClient = FindClient(sAValue);

				if (pClient == NULL)
				{
					(*pBookI)->ClearBorrower();
					continue;
				}

				(*pBookI)->InitBorrower(reinterpret_cast<Item*>(pClient));
			}
		}
	}

	//_[output load results]__________________________________________________
	cout << "Loaded " << iBooks << " book(s) and " << iClients << " client(s) from \"" << sFilename << "\"" << endl;

	//_[return successful]____________________________________________________
	return Okay;
}

//_[BookShop::SaveXML()]______________________________________________________
//
// DESCRIPTION:
//  Saves all books and clients to specified XML file.
//
// PARAMETERS:
//  sFilename ............. name of XML file to save to
//
BookShop::return_value BookShop::SaveXML(string &sFilename)
{
	fstream oFile;				// file stream object
	string sName, sValue;		// name/value attribute pair
	unsigned int i;				// general use index variable

	//_[attempt to open output file]__________________________________________
	oFile.open(sFilename.c_str(), ios::out);

	//_[output root tag]______________________________________________________
	oFile << "<BOOKSHOP>" << endl << endl;

	//_[output book collection]_______________________________________________
	if (aBooks.size())
	{
		vector<Book*>::iterator pBook;

		oFile << "\t<BOOKS>" << endl;

		for (pBook = aBooks.begin(); pBook != aBooks.end(); pBook++)
		{
			oFile << "\t\t<BOOK ";
			for (i = 0; i < (*pBook)->numProps(); i++)
			{
				sName = (*pBook)->getName(i);
				sValue = (*pBook)->getValue(sName);
				oFile << sName << "=\"" << stringFlatten(sValue) << "\" ";
			}
			oFile << "/>" << endl;
		}

		oFile << "\t</BOOKS>" << endl;
	}

	//_[output client collection]_____________________________________________
	if (aClients.size())
	{
		vector<Client*>::iterator pClient;

		oFile << "\t<CLIENTS>" << endl;

		for (pClient = aClients.begin(); pClient != aClients.end(); pClient++)
		{
			oFile << "\t\t<CLIENT ";
			for (i = 0; i < (*pClient)->numProps(); i++)
			{
				sName = (*pClient)->getName(i);
				sValue = (*pClient)->getValue(sName);
				oFile << sName << "=\"" << stringFlatten(sValue) << "\" ";
			}
			oFile << "/>" << endl;
		}

		oFile << "\t</CLIENTS>" << endl;
	}

	//_[output root cancel tag]_______________________________________________
	oFile << endl << "</BOOKSHOP>";

	//_[close output file]____________________________________________________
	oFile.close();

	//_[output results]_______________________________________________________
	cout << "Saved " << aBooks.size() << " book(s) and " << aClients.size() << " clients to \"" << sFilename << "\"" << endl;

	//_[result successful]____________________________________________________
	return Okay;
}

//_[BookShop::ShowBook()]_____________________________________________________
//
// DESCRIPTION:
//  Shows all information about a particular book.
//
// PARAMETERS:
//  sBook ................. id of book to show info about
//
BookShop::return_value BookShop::ShowBook(string &sBook)
{
	Book *pBook = FindBook(sBook);				// pointer to book
	string sName;								// general name string

	//_[if book doesn't exist, return error]__________________________________
	if (pBook == NULL)
		return BookNoExist;

	//_[show all properties]__________________________________________________
	for (unsigned int i = 0; i < pBook->numProps(); i++)
	{
		sName = pBook->getName(i);
		cout << sName << ": " << pBook->getValue(sName) << endl;
	}

	//_[show if book is overdue]______________________________________________
	if (pBook->isDue())
		cout << "(overdue)" << endl;

	//_[return successful]____________________________________________________
	return Okay;
}

//_[BookShop::ShowClient()]___________________________________________________
//
// DESCRIPTION:
//  Display details of specified client.
//
// PARAMETERS:
//  sClient ............... id of client to show details of
//
BookShop::return_value BookShop::ShowClient(string &sClient)
{
	Client *pClient = FindClient(sClient);	// pointer to client
	string sName;							// general name string
	unsigned int i = 0;						// general index variable

	//_[if client doesn't exist, return error]________________________________
	if (pClient == NULL)
		return ClientNoExist;

	//_[show all client's properties]_________________________________________
	for (i = 0; i < pClient->numProps(); i++)
	{
		sName = pClient->getName(i);
		cout << sName << ": " << pClient->getValue(sName) << endl;
	}

	//_[if the client has books borrowed]_____________________________________
	if (pClient->numBorrowed())
	{
		Book *pBook;

		cout << "Books borrowed: " << endl;
		for (i = 0; i < pClient->numBorrowed(); i++)
		{
			//_[get pointer to book]__________________________________________
			pBook = reinterpret_cast<Book*>(pClient->getBorrowed(i));

			//_[get name of id property of book]______________________________
			sName = pBook->getIDProp();

			//_[output index of book to client]_______________________________
			cout << setw(4) << i << ": ";

			//_[output book identifier]_______________________________________
			if (pBook == NULL)
			{
				 cout << "Non-existant book";		// bad book pointer
			}
			else
			{
				cout << pBook->getValue(sName);		// book's identifier

				//-[show if book is overdue]----------------------------------
				if (pBook->isDue())
					cout << " (overdue)" << endl;
			}
		}
	}

	//_[return successful]____________________________________________________
	return Okay;
}

//_[BookShop::NewItem()]______________________________________________________
//
// DESCRIPTION:
//  Generic function for a new item: either a book or client.  Gets user input
//  for each property.
//
// PARAMETERS:
//  pItem ................. pointer to new item
//
BookShop::return_value BookShop::NewItem(Item *pItem)
{
	unsigned int i;
	string sName, sValue;
	bool fUnset;

	for (i = 0; i < pItem->numProps(); i++)
	{
		sName = pItem->getName(i);
		cout << pItem->getDesc(sName) << endl << endl;
		fUnset = true;

		do
		{
			cout << sName << ": ";
			getline(cin, sValue);

			if (pItem->setProp(sName, sValue) != Item::Okay)
			{
				cout << "Could not set '" << sName << "' to input specified.  Please follow the rules described." << endl;
			}
			else
			{
				fUnset = false;
			}
		} while (fUnset);
	}

	return Okay;
}

//_[BookShop::FindBook()]_____________________________________________________
//
// DESCRIPTION:
//  Finds a book in the vector by the specified id.
//
// PARAMETERS:
//  sBook ................. id of book to find
//
// RETURN VALUE (Book*):
//  Returns to pointer to Book object, or NULL if it wasn't found.
//
Book *BookShop::FindBook(string &sBook)
{
	vector<Book*>::iterator pBook;
	string sName;

	if (!aBooks.size())
		return NULL;

	sName = aBooks[0]->getIDProp();

	for (pBook = aBooks.begin(); pBook != aBooks.end(); pBook++)
	{
		if ((*pBook)->getValue(sName) == sBook)
		{
			return *pBook;
		}
	}

	return NULL;
}

//_[BookShop::FindClient()]___________________________________________________
//
// DESCRIPTION:
//  Finds a client in the vector by the specified id.
//
// PARAMETERS:
//  sClient ............... id of book to find
//
// RETURN VALUE (Client*):
//  Returns to pointer to Client object, or NULL if it wasn't found.
//
Client *BookShop::FindClient(string &sClient)
{
	vector<Client*>::iterator pClient;
	string sName;

	if (!aClients.size())
		return NULL;

	sName = aClients[0]->getIDProp();

	for (pClient = aClients.begin(); pClient != aClients.end(); pClient++)
	{
		if ((*pClient)->getValue(sName) == sClient)
		{
			return *pClient;
		}
	}

	return NULL;
}

//_[BookShop::getError()]_____________________________________________________
//
// DESCRIPTION:
//  Returns a string representation of the return code specified.
//
// PARAMETERS:
//  return_value .......... return code
//
// RETURN VALUE (string):
//  See description.
//
string BookShop::getError(BookShop::return_value iErr)
{
	string sError;

	switch (iErr)
	{
		case Okay:
			sError = "Operation succeeded";
			break;
		case Error:
			sError = "General operation error";
			break;
		case NoClients:
			sError = "There are currently no known clients";
			break;
		case NoBooks:
			sError = "There are currently no known books";
			break;
		case NoMatch:
			sError = "A match was not found by the criteria you specified";
			break;
		case ClientNoExist:
			sError = "The specified client does not exist";
			break;
		case BookNoExist:
			sError = "The specified book does not exist";
			break;
		case BookOut:
			sError = "The book specified is already being borrowed";
			break;
		case FileNotFound:
			sError = "The specified file was not found";
			break;
		case FileCorrupt:
			sError = "The specified file was corrupted";
			break;
	}

	return sError;
}
