/*****************************************************************************

	[bookshop.h] BookShop class prototype
	By Jeff Leung

*****************************************************************************/
#ifndef BOOKSHOP_H
#define BOOKSHOP_H

#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <cstring>
#include <vector>
#include "client.h"
#include "book.h"

using namespace std;

class BookShop
{
public:
	BookShop();
	~BookShop();

	//_[function return values]_______________________________________________
	enum return_value
	{
		Okay,				// function succeeded
		Error,				// general error
		NoClients,			// there are no clients
		NoBooks,			// there are no books
		NoMatch,			// no match for specified criteria
		ClientNoExist,		// client specified does not exist
		BookNoExist,		// book specified does not exist
		BookOut,			// book is currently out being borrowed
		FileNotFound,		// specified file was not found
		FileCorrupt			// specified file was corrupt
	};

	//_[filter types]_________________________________________________________
	enum filter_type
	{
		None			= 0x0000,	//
		All 			= 0x0077,	// all
		AllBooks		= 0x0007,	// all books
		InBooks			= 0x0001,	// in books
		OutBooks		= 0x0006,	// borrowed books
		NotDueBooks		= 0x0002,	// not due borrowed books
		DueBooks		= 0x0004,	// due borrowed books
		AllClients		= 0x0070,	// all clients
		EmptyClients	= 0x0010,	// clients not borrowing
		BookClients		= 0x0060,	// clients borrowing
		GoodClients		= 0x0020,	// clients borrowing with no books due
		BadClients		= 0x0040	// clients borrowing with books due
	};

	return_value Tally(filter_type);
	return_value List(filter_type,unsigned int);
	return_value NewBook(void);
	return_value NewClient(void);
	return_value DeleteBook(string&);
	return_value DeleteClient(string&);
	return_value Borrow(string&,string&);
	return_value Return(string&,string&);
	return_value LoadXML(string&);
	return_value SaveXML(string&);
	return_value ShowBook(string&);
	return_value ShowClient(string&);
	string getError(return_value);

protected:
private:
	Book *FindBook(string &);
	Client *FindClient(string &);
	return_value NewItem(Item *);

	vector<Client*>	aClients;
	vector<Book*>	aBooks;
};

#endif