/*****************************************************************************

	[client.cpp] Client class implementation
	By Jeff Leung

*****************************************************************************/
#include "client.h"
#include "book.h"

//_[Client::Client()]_________________________________________________________
//
// DESCRIPTION:
//  Default constructor; does nothing.
//
// PARAMETERS:
//  None.
//
Client::Client()
{
}

//_[Client::~Client()]________________________________________________________
//
// DESCRIPTION:
//  Destructor; does nothing.
//
Client::~Client()
{
}

//_[Client::getIDProp()]______________________________________________________
//
// DESCRIPTION:
//  See Item::getIDProp().
//
string Client::getIDProp(void)
{
	string sIDProp = "ID";
	return sIDProp;
}

//_[Client::getName()]________________________________________________________
//
// DESCRIPTION:
//  See Item::getName().
//
string Client::getName(unsigned int iProp)
{
	string sName;

	switch (iProp)
	{
		case 0:		sName = "ID";			break;
		case 1:		sName = "FIRSTNAME";	break;
		case 2:		sName = "LASTNAME";		break;
	}

	return sName;
}

//_[Client::getValue()]_______________________________________________________
//
// DESCRIPTION:
//  See Item::getValue().
//
string Client::getValue(string &sProp)
{
	if (sProp == "ID")
		return sID;
	else if (sProp == "FIRSTNAME")
		return sFirstName;
	else if (sProp == "LASTNAME")
		return sLastName;
}

//_[Client::getDesc()]________________________________________________________
//
// DESCRIPTION:
//  See Item::getDesc().
//
string Client::getDesc(string &sProp)
{
	string sDesc;

	if (sProp == "ID")
		sDesc = "Unique identifier for client";
	else if (sProp == "FIRSTNAME")
		sDesc = "Client's first name";
	else if (sProp == "LASTNAME")
		sDesc = "Client's last name";

	return sDesc;
}

//_[Client::setProp()]________________________________________________________
//
// DESCRIPTION:
//  See Item::setProp().
//
Item::return_value Client::setProp(string &sProp, string &sValue)
{
	if (sProp == "ID")
		sID = sValue;
	else if (sProp == "FIRSTNAME")
		sFirstName = sValue;
	else if (sProp == "LASTNAME")
		sLastName = sValue;

	return Item::Okay;
}

//_[Client::numProps()]_______________________________________________________
//
// DESCRIPTION:
//  See Item::numProps().
//
unsigned int Client::numProps(void)
{
	return 3;
}

//_[Client::Borrow()]_________________________________________________________
//
// DESCRIPTION:
//  Attempts to borrow a particular book.  First the book has to be available,
//  then the pointer to the book is stored in the client's internal vector.
//  The client initiates the borrow, but when data is reloaded (as from a 
//  file), the book is the one who reinstates the link.
//
// PARAMETERS:
//  pBook ................. pointer to book being borrowed
//  sDate ................. due date of book
//
// RETURN VALUE(bool):
//  true on success, false on failure.
//
bool Client::Borrow(Item *pBook, string &sDate)
{
	if (reinterpret_cast<Book*>(pBook)->SetBorrower(reinterpret_cast<Item*>(this), sDate) == false)
		return false;

	aBorrow.push_back(reinterpret_cast<Item*>(pBook));
	return true;
}

//_[Client::Return()]_________________________________________________________
//
// DESCRIPTION:
//  Attempts to return a book that has been borrowed.  This means destroying
//  the link between the book and the client.  This is initiated by the client
//  and wrapped up by the book.  The client also removes it's record of the
//  book from it's internal vector.
//
// PARAMETERS:
//  pBook ................. pointer to book being returned
//
// RETURN VALUE(bool):
//  true on success, false on failure.
//
bool Client::Return(Item *pBook)
{
	vector<Item*>::iterator pBookI;

	for (pBookI = aBorrow.begin(); pBookI != aBorrow.end(); pBookI++)
	{
		if (*pBookI == pBook)
		{
			reinterpret_cast<Book*>(pBook)->ClearBorrower();
			aBorrow.erase(pBookI, pBookI+1);
			return true;
		}
	}
	
	return false;
}

//_[Client::InitBorrowed()]___________________________________________________
//
// DESCRIPTION:
//  When the link between a book borrowed and a client must be restored (as
//  when loading data from a file), this function is called by the BOOK.  The
//  client merely acknowledges the link and restores it on this side, but the
//  book is what initiates the link.
//
// PARAMETERS:
//  pBook ................. pointer to book being re-borrowed
//
// RETURN VALUE(bool):
//  true on success, false on failure.
//
bool Client::InitBorrowed(Item *pBook)
{
	unsigned int i;

	for (i = 0; i < aBorrow.size(); i++)
	{
		if (aBorrow[i] == pBook)
			break;
	}

	if (i >= aBorrow.size())
		aBorrow.push_back(pBook);
}

//_[Client::isBorrowing()]____________________________________________________
//
// DESCRIPTION:
//  Returns true if the client is borrowing any books.
//
// PARAMETERS:
//  None.
//
// RETURN VALUE(bool):
//  See description.
//
bool Client::isBorrowing(void)
{
	if (aBorrow.size())
		return true;
	return false;
}

//_[Client::hasDue()]_________________________________________________________
//
// DESCRIPTION:
//  Returns true if the client has any books due.
//
// PARAMETERS:
//  None.
//
// RETURN VALUE(bool):
//  See description.
//
bool Client::hasDue(void)
{
	vector<Item*>::iterator pBookI;
	bool fDue = false;

	if (!aBorrow.size())
		return false;

	for (pBookI = aBorrow.begin(); pBookI != aBorrow.end() && !fDue; pBookI++)
	{
		fDue = reinterpret_cast<Book*>(*pBookI)->isDue();
	}

	return fDue;
}

//_[Client::numBorrowed()]____________________________________________________
//
// DESCRIPTION:
//  Returns the number of books being borrowed by this client.
//
// PARAMETERS:
//  None.
//
// RETURN VALUE (unsigned int):
//  See description.
//
unsigned int Client::numBorrowed(void)
{
	return aBorrow.size();
}

//_[Client::getBorrowed()]____________________________________________________
//
// DESCRIPTION:
//  Returns a pointer to a book that is being borrowed.
//
// PARAMETERS:
//  iBook ................. index to borrowed books vector
//
// RETURN VALUE (Item*):
//  See description; returns NULL on error.
//
Item *Client::getBorrowed(unsigned int iBook)
{
	if (iBook > aBorrow.size())
		return NULL;

	return aBorrow[iBook];
}
