/*****************************************************************************

	[main.cpp] Book Shop main program module
	By Jeff Leung

*****************************************************************************/
#include <cstring>
#include <vector>
#include <string>
#include <iostream>
#include <stdlib.h>
#include "bookshop.h"
#include "help.h"
#include "misc.h"

using namespace std;

//_[main()]___________________________________________________________________
//
// DESCRIPTION:
//
// PARAMETERS:
//
// RETURN VALUE():
//
int main()
{
	BookShop b;
	string sCmd;
	vector<string> aWords;
	string::size_type iPos, iLen;
	bool fErrored;
	BookShop::return_value iError;
	unsigned int i;

	//_[output program information header]____________________________________
	//
	//
	cout
		<< "Book Shop" << endl
		<< "By Jeff Leung" << endl
		<< endl
		<< "Enter a command at the prompt or 'help' for assistance"
		<< endl;

	//_[main command loop]____________________________________________________
	//
	//
	do
	{
		fErrored = false;
		iError = BookShop::Okay;
		cout << "> ";
		getline(cin, sCmd);
		if (sCmd == "")
			continue;

		//-[separate command into it's separate words]------------------------
		aWords.clear();
		for (iPos = 0; iPos < sCmd.length(); iPos = iPos + iLen + 1)
		{
			iLen = sCmd.find(" ", iPos);
			if (iLen == string::npos)
				iLen = sCmd.length();
			iLen -= iPos;
			aWords.push_back(sCmd.substr(iPos,iLen));
		}

		i = nextWord(aWords, 0);

		//-[tally]------------------------------------------------------------
		//
		// USAGE:
		//  tally <filter>
		//
		if (aWords[i] == "tally")
		{
			BookShop::filter_type iFilter;
			i = nextWord(aWords, i+1);

			iFilter = (BookShop::filter_type)getFilter(aWords[i]);

			if (!iFilter)
			{
				cout << "None of the filters were accepted.  Type 'help filters' for more information on filters." << endl;
				i = aWords.size();
				fErrored = true;
			}

			if (!fErrored)
				iError = b.Tally(iFilter);
		}
		//-[load]-------------------------------------------------------------
		//
		// USAGE:
		//  load <filename>.xml
		//
		else if (aWords[i] == "load")
		{
			i = nextWord(aWords, i+1);

			if (i)
				iError = b.LoadXML(aWords[i]);
			else
				fErrored = true;
		}
		//-[save]-------------------------------------------------------------
		//
		// USAGE:
		//  save <filename>.xml
		//
		else if (aWords[i] == "save")
		{
			i = nextWord(aWords, i+1);

			if (i)
				iError = b.SaveXML(aWords[i]);
			else
				fErrored = true;
		}
		//-[list]-------------------------------------------------------------
		//
		// USAGE:
		//  list <filter> <start> <end>
		//
		else if (aWords[i] == "list")
		{
			BookShop::filter_type iFilter = BookShop::All;
			unsigned int iStart;

			i = nextWord(aWords, i+1);
			if (!i)
			{
				fErrored = true;
			}
			else
			{
				iFilter = (BookShop::filter_type)getFilter(aWords[i]);
				if (iFilter == 0)
				{
					i = aWords.size();
					fErrored = true;
				}
			}

			if (!fErrored)
			{
				i = nextWord(aWords, i+1);

				if (!i)
					iStart = 0;
				else
					iStart = strtol(aWords[i].c_str(), NULL, 0);

				iError = b.List(iFilter, iStart);
			}
		}
		//-[show]-------------------------------------------------------------
		//
		// USAGE:
		//  show ("book"/"client") (<book>/<client>)
		//
		else if (aWords[i] == "show")
		{
			i = nextWord(aWords, i+1);
			if (aWords[i] == "book")
			{
				i = nextWord(aWords, i+1);
				if (i)
					iError = b.ShowBook(aWords[i]);
				else
					fErrored = true;
			}
			else if (aWords[i] == "client")
			{
				i = nextWord(aWords, i+1);
				if (i)
					iError = b.ShowClient(aWords[i]);
				else
					fErrored = true;
			}
			else
			{
				fErrored = true;
			}
		}
		//-[new]--------------------------------------------------------------
		//
		// USAGE:
		//  new ("book"/"client")
		//
		else if (aWords[i] == "new")
		{
			i = nextWord(aWords, i+1);

			if (aWords[i] == "book")
				iError = b.NewBook();
			else if (aWords[i] == "client")
				iError = b.NewClient();
			else
				fErrored = true;
		}
		//-[delete]-----------------------------------------------------------
		//
		// USAGE:
		//  delete ("book"/"client") (<book>/<client>)
		//
		else if (aWords[i] == "delete")
		{
			i = nextWord(aWords, i+1);

			if (aWords[i] == "book")
			{
				i = nextWord(aWords, i+1);
				
				if (i)
					iError = b.DeleteBook(aWords[i]);
				else
					fErrored = true;
			}
			else if (aWords[i] == "client")
			{
				i = nextWord(aWords, i+1);
				
				if (i)
					iError = b.DeleteClient(aWords[i]);
				else
					fErrored = true;
			}
			else
			{
				fErrored = true;
			}
		}
		//-[borrow]-----------------------------------------------------------
		//
		// USAGE:
		//  borrow <book> <client>
		//
		else if (aWords[i] == "borrow")
		{
			unsigned int iBook = i = nextWord(aWords, i+1);

			if (i)
			{
				i = nextWord(aWords, i+1);

				if (i)
					iError = b.Borrow(aWords[iBook], aWords[i]);
				else
					fErrored = true;
			}
			else
			{
				fErrored = true;
			}
		}
		//-[return]-----------------------------------------------------------
		//
		// USAGE:
		//  return <book> <client>
		//
		else if (aWords[i] == "return")
		{
			unsigned int iBook = i = nextWord(aWords, i+1);

			if (i)
			{
				i = nextWord(aWords, i+1);

				if (i)
					iError = b.Return(aWords[iBook], aWords[i]);
				else
					fErrored = true;
			}
			else
			{
				fErrored = true;
			}
		}
		//-[help]-------------------------------------------------------------
		//
		// USAGE:
		//  help [command]
		//
		else if (aWords[i] == "help")
		{
			printHelp(aWords, i);
		}
		//-[quit]-------------------------------------------------------------
		//
		//
		else if (aWords[i] == "quit")
		{
		}
		//-[unknown command]--------------------------------------------------
		//
		//
		else
		{
			cout << "Unknown command.  Type 'help' for a list of valid commands." << endl;
		}

		//-[if a syntax error occurred]---------------------------------------
		if (fErrored)
		{
			if (!i)
				cout << "Insufficient parameters.";
			else if (i == aWords.size())
				cout << "Parameters were not accepted.";
			else
				cout << "Unknown parameter '" << aWords[i] << "'.";

			cout << "  Please type 'help " << aWords[nextWord(aWords,0)] << "' for usage information" << endl;
		}

		//-[if a function error occurred]-------------------------------------
		if (iError != BookShop::Okay)
		{
			cout << "ERROR: " << b.getError(iError) << endl;
		}


	} while (sCmd != "quit");

	return 0;
}