//file: library.cc

#include "library.h"

Library :: Library(ifstream &inFile)
: bookcount (0)
, cardcount (0)
, bindcount (0)
{
   cout << "Initializing library database ... " << endl;

   cout << "      initializing book list:" << endl;

   while (bookcount < Max_Len &&
         ReadBook(bookList[bookcount], inFile) == True)
   {
      cout << "      processing book - " << bookList[bookcount] << endl;
      bookcount++;
   }

   cout << bookcount << " books in library" << endl;

   cout << "      checking for books @ bindery:" << endl;

   //
   // restoring data from 'library.data' restores links, but not bindery
   // information which cannot be stored in the current format.
   //
   for (int i = 0; bindcount < Max_AtBindery && i < bookcount; i++)
   {
      if (bookList[i].GetStatus() == -1)
      {
         bindList[bindcount].Set(bookList[i].GetNumber(), "", 0.0);
         bindList[bindcount].Sent.Set(0,0,0);
         bindList[bindcount].Returned.Set(0,0,0);
         bindcount++;
         cout << "      found - " << bookList[i] << endl;
      }
   }

   cout << "      initializing card list:" << endl;

   while (cardcount < Max_Len &&
         ReadCard(cardList[cardcount], inFile) == True)
   {
      cout << "      processing card- " << cardList[cardcount] << endl;
      cardcount++;
   }

   cout << cardcount << " cards in library" << endl;

   // close the input file (since 'main()' doesn't) so we can write out the
   // file when we're finished
   inFile.close();

   cout << "Done initializing Library Database." << endl;
}

Library :: ~Library()
{
   int i;
   ofstream outFile("library.data");

   cout << "saving library database to file ... " << endl;

   for (i = 0; i < bookcount; i++)
   {
      bookList[i].Write(outFile);
   }

   outFile << "$$$" << endl;

   for (i = 0; i < cardcount; i++)
   {
      cardList[i].Write(outFile);
   }

   if (outFile.fail())
   {
      cout << "database file (library.data) update failed ... " << endl;
   }
   else
   {
      cout << "database file (library.data) successfully updated ... " << endl;
   }

   outFile.close();

   cout << " ... deleting library database ... " << endl;
}

void Library :: DoCommand(char cmd)
{
   switch (cmd)
   {
      case 'C':
         ShowAllCards();
         break;

      case 'B':
         ShowAllBooks();
         break;

      case 'O':
         CheckOutBook();
         break;

      case 'I':
         CheckInBook();
         break;

      case 'A':
         AddBook();
         break;

      case 'N':
         NewCard();
         break;

      case 'P':
         PrintBook();
         break;

      case 'S':
         SendToBindery();
         break;
   }
}

Bool Library :: ReadBook(Book &book, ifstream &inFile)
{
   char title[4096], author[4096];
   int number, status, owner;

   inFile.getline(title, 4096);

   // might get left-overs from last read, check to make sure we have a non-
   // whitespace/blank title
   if (title[0] <= 32)
      inFile.getline(title, 4096);

   // if the line is '$$$' then it means no more books are in data file
   if (!strcmp(title, "$$$"))
      return False;

   // otherwise the line is the title (keep)

   // get author
   inFile.getline(author, 4096);

   // get other numbers
   inFile >> number >> status >> owner;

   // initialize book with data
   book.Init(title, author, number);
   book.SetStatus(status);
   book.SetOwner(owner);

   return True;
}

Bool Library :: ReadCard(Card &card, ifstream &inFile)
{
   char name[4096], phone[4096];
   int id, book;

   inFile.getline(name, 4096);

   // might get left-overs from last read, check to make sure we have a non-
   // whitespace/blank name
   if (name[0] <= 32)
      inFile.getline(name, 4096);

   // if at end of file then there are no more cards
   if (inFile.eof() || inFile.fail())
      return False;

   // otherwise the line is the name (keep)

   // get phone number
   inFile.getline(phone, 4096);

   // get other numbers
   inFile >> id >> book;

   // initialize card with data
   card.Init(name, phone, id);
   card.SetBook(book);

   return True;
}

void Library :: ShowAllBooks()
{
   int i;
   for (i = 0; i < bookcount; i++)
   {
      cout << bookList[i] << endl;
   }
}

void Library :: ShowAllCards()
{
   int i;
   for (i = 0; i < cardcount; i++)
   {
      cout << cardList[i] << endl;
   }
}

void Library :: CheckOutBook()
{
   int ibook, icard, booknum, cardid;

   cout << "Enter book #: ";
   cin >> booknum;

   // find book in array
   for (ibook = 0; ibook < bookcount; ibook++)
   {
      if (bookList[ibook].GetNumber() == booknum)
         break;
   }

   if (ibook < bookcount)
   {
      if (bookList[ibook].GetStatus() == 0)
      {
         cout << "Enter your card #: ";
         cin >> cardid;

         for (icard = 0; icard < cardcount; icard++)
         {
            if (cardList[icard].GetID() == cardid)
               break;
         }

         if (icard < cardcount)
         {
            if (cardList[icard].GetBook() == 0)
            {
               bookList[ibook].SetStatus(1);
               bookList[ibook].SetOwner(cardid);
               cardList[icard].SetBook(booknum);

               cout << "Check-out succeeded: " << endl
                     << "Book: " << bookList[ibook] << endl
                     << "Card: " << cardList[icard] << endl;
            }
            else
            {
               cout << "Card (" << cardList[icard] << ") still has book #"
                    << cardList[icard].GetBook() << " checked out!" << endl;
            }
         }
         else
         {
            cout << "Card #" << cardid << " not found!" << endl;
         }
      }
      else if (bookList[ibook].GetStatus() == -1)
      {
         cout << "Book (" << bookList[ibook] << ") still at bindery!" << endl;
      }
      else
      {
         cout << "Book (" << bookList[ibook]
              << ") already checked out with card #"
              << bookList[ibook].GetOwner() << "!" << endl;
      }
   }
   else
   {
      cout << "Book #" << booknum << " not found!" << endl;
   }
}

void Library :: CheckInBook()
{
   int ibook, icard, booknum;

   cout << "Enter book #:";
   cin >> booknum;

   // find book in array
   for (ibook = 0; ibook < bookcount; ibook++)
   {
      if (bookList[ibook].GetNumber() == booknum)
         break;
   }

   if (ibook < bookcount)
   {
      if (bookList[ibook].GetStatus() == 1)
      {
         for (icard = 0; icard < cardcount; icard++)
         {
            if (cardList[icard].GetID() == bookList[ibook].GetOwner())
               break;
         }

         if (icard < cardcount)
         {
            if (cardList[icard].GetBook() != 0)
            {
               bookList[ibook].SetStatus(0);
               bookList[ibook].SetOwner(0);
               cardList[icard].SetBook(0);

               cout << "Check-in succeeded: " << endl
                     << "Book: " << bookList[ibook] << endl
                     << "Card: " << cardList[icard] << endl;
            }
            else
            {
               cout << "Card record does not show book checked out!" << endl;
               cout << "Clearing book check-out status anyway!" << endl;
               bookList[ibook].SetStatus(0);
               bookList[ibook].SetOwner(0);
            }
         }
         else
         {
            cout << "Card #" << bookList[ibook].GetOwner() << " not found!"
                  << endl;
            cout << "Clearing book check-out status anyway!" << endl;
            bookList[ibook].SetStatus(0);
            bookList[ibook].SetOwner(0);
         }
      }
      else if (bookList[ibook].GetStatus() == -1)
      {
         cout << "Book (" << bookList[ibook] << ") still at bindery!" << endl;
      }
      else
      {
         cout << "Book (" << bookList[ibook] << ") not checked out!" << endl;
      }
   }
   else
   {
      cout << "Book #" << booknum << " not found!" << endl;
   }
}

void Library :: AddBook()
{
   if (bookcount < Max_Len)
   {
      char title[4096], author[4096];
      int num = 0, i;

      cout << "Enter book title: ";
      cin.getline(title, 4096);

      // if there are left-overs in the input stream, it sometimes causes an
      // instantaneous "blank" title
      if (title[0] <= 32)
         cin.getline(title, 4096);

      cout << "Enter book author: ";
      cin.getline(author, 4096);

      // look for last largest 'num' (to make it one size larger)
      for (i = 0; i < bookcount; i++)
      {
         if (bookList[i].GetNumber() >= num)
            num = bookList[i].GetNumber() + 1;
      }

      // initialize new card
      bookList[bookcount].Init(title, author, num);
      bookList[bookcount].SetStatus(0);

      cout << "Created book: " << bookList[bookcount] << endl;

      bookcount++;
   }
   else
   {
      cout << "No more room in database to create new book!" << endl;
   }
}

void Library :: NewCard()
{
   if (cardcount < Max_Len)
   {
      char name[4096], phone[4096];
      int id = 0, i;

      cout << "Enter your name: ";
      cin.getline(name, 4096);

      // if there are left-overs in the input stream, it sometimes causes an
      // instantaneous "blank" name
      if (name[0] <= 32)
         cin.getline(name, 4096);

      cout << "Enter your phone number: ";
      cin.getline(phone, 4096);

      // look for last largest 'id' (to make it one size larger)
      for (i = 0; i < cardcount; i++)
      {
         if (cardList[i].GetID() >= id)
            id = cardList[i].GetID() + 1;
      }

      // initialize new card
      cardList[cardcount].Init(name, phone, id);

      cout << "Created card: " << cardList[cardcount] << endl;

      cardcount++;
   }
   else
   {
      cout << "No more room in database to create new card!" << endl;
   }
}

void Library::PrintBook()
{
   int icard, ibook, booknum;
   cout << "Enter book #:";
   cin >> booknum;

   // find book in array
   for (ibook = 0; ibook < bookcount; ibook++)
   {
      if (bookList[ibook].GetNumber() == booknum)
         break;
   }

   if (ibook < bookcount)
   {
      cout << bookList[ibook] << endl;

      // checked out ...
      if (bookList[ibook].GetStatus() == 1)
      {
         for (icard = 0; icard < cardcount; icard++)
         {
            if (cardList[icard].GetID() == bookList[ibook].GetOwner())
               break;
         }

         if (icard < cardcount)
         {
            cout << "Checked out with card: " << cardList[icard] << endl;
         }
      }
      else if (bookList[ibook].GetStatus() == -1)
      {
         for (ibook = 0; ibook < bindcount; ibook++)
         {
            if (bindList[ibook].GetBook() == booknum)
               break;
         }

         if (ibook < bindcount)
         {
            cout << "Sent to bindery on " << bindList[ibook].Sent << ", back on " <<
               bindList[ibook].Returned << endl;
            cout << "Reason at bindery: \"" << bindList[ibook].GetCondition()
               << "\", Cost: " << bindList[ibook].GetCost() << endl;
         }
      }
      else
      {
         cout << "Book is on the shelf." << endl;
      }
   }
}

void Library::SendToBindery()
{
   int ibook, booknum;
   char condition[16] = {0};
   double cost;

   if (bindcount == Max_AtBindery)
   {
      cout << "Can't send any more books to the bindery!" << endl;
      return;
   }

   cout << "Enter book #:";
   cin >> booknum;

   // find book in array
   for (ibook = 0; ibook < bookcount; ibook++)
   {
      if (bookList[ibook].GetNumber() == booknum)
         break;
   }

   if (ibook < bookcount)
   {
      // book must be on the shelf
      if (bookList[ibook].GetStatus() == 0)
      {
         cout << "Enter condition (reason - 16 char max): ";

         // don't let condition be blank
         do
         {
            cin.getline(condition, 16);
         } while (condition[0] == '\0');

         cout << "Enter cost: ";
         cin >> cost;

         // set information gathered thus far
         bindList[bindcount].Set(booknum, condition, cost);

         cout << "Enter date sent:" << endl;
         bindList[bindcount].Sent.GetInput(cout, cin);

         cout << "Enter date to be returned back:" << endl;
         bindList[bindcount].Returned.GetInput(cout, cin);

         while (!bindList[bindcount].Returned.higherThan(bindList[bindcount].Sent))
         {
            cout << "The received date must be LATER than the sent date!" << endl
               << "Please enter the date to be returned back again:" << endl;
            bindList[bindcount].Returned.GetInput(cout, cin);
         }

         // set book status
         bookList[ibook].SetStatus(-1);

         cout << "Book successfully sent to bindery to return on "
               << bindList[bindcount].Returned << "!" << endl;

         // add to counter of books at bindery (bindList used size)
         bindcount++;
      }
      else
      {
         cout << "Book is gone!  It can't be sent to the bindery!" << endl;
      }
   }
   else
   {
      cout << "Book #" << booknum << " not found!" << endl;
   }
}