DECLARE SUB DrawDrops ()
DECLARE SUB DefScene (xcoord AS INTEGER, ycoord AS INTEGER, letter AS STRING, col AS INTEGER, dn AS INTEGER)

DECLARE SUB CreateTeap (xcoord AS INTEGER, ycoord AS INTEGER)
DECLARE SUB BuildAcharTable ()
DECLARE SUB Initialize ()
DECLARE SUB CreateDrops (darray() AS ANY, dnum!)

DECLARE FUNCTION AcceptableChar% ()
DECLARE SUB ini.timer (fps AS LONG)
DECLARE SUB do.timer ()

'
' The QB-Matrix
' Neil C. Obremski - 27 APR 1999
'

CONST TIMER.PRECISION = 2
CONST DROP.MODE.CLEAR = 1
CONST DROP.MODE.DISPLAY = 2
CONST BACK.MODE.FLIP = 1
CONST BACK.MODE.CLEAR = 2

'
' color constants
'
CONST BLACK = 0
CONST BLUE = 1
CONST GREEN = 2
CONST CYAN = 3
CONST RED = 4
CONST PURPLE = 5
CONST BROWN = 6
CONST WHITE = 7
CONST GRAY = 8
CONST BBLUE = 9
CONST BGREEN = 10
CONST BCYAN = 11
CONST BRED = 12
CONST BPURPLE = 13
CONST YELLOW = 14
CONST BWHITE = 15

CONST FALSE = 0
CONST TRUE = 1

  TYPE DROP
    x AS INTEGER      ' horizontal
    y AS INTEGER      ' vertical
    tm AS INTEGER     ' timer max
    t AS INTEGER      ' timer
    m AS INTEGER      ' mode
    ep AS INTEGER     ' end point (vertical)
  END TYPE

  TYPE teap           ' title piece (letter)
    x AS INTEGER      ' horizontal (in scene table)
    y AS INTEGER      ' vertical (in scene table)
    cd AS INTEGER     ' countdown
  END TYPE

  TYPE backscreen
    c AS INTEGER      ' character to show
    active AS INTEGER ' active?
    dc AS INTEGER     ' display color
    m AS INTEGER      ' mode of display (flip/clear)
  END TYPE

DIM SHARED ach(256) AS INTEGER, nach AS INTEGER
DIM SHARED cps AS LONG, stoptime AS INTEGER  ' counts per second
DIM SHARED scene(80, 24) AS backscreen
DIM SHARED ntep AS INTEGER, tep(25) AS teap
DIM SHARED darray(256) AS DROP, dnum AS INTEGER

  dnum = 50

  Initialize

  WHILE (INKEY$ = "")
    DrawDrops
    do.timer
  WEND

  END

Finish:
  stoptime = 1
  RETURN

FUNCTION AcceptableChar%

  AcceptableChar = ach(INT(RND * nach))

END FUNCTION

SUB CreateTeap (xcoord AS INTEGER, ycoord AS INTEGER)

  IF (scene(xcoord, ycoord).active = 0) THEN
    tep(ntep).x = xcoord
    tep(ntep).y = ycoord
    tep(ntep).cd = INT(RND * 40) + 1
    ntep = ntep + 1
    scene(xcoord, ycoord).active = 1
  END IF

END SUB

SUB DefScene (xcoord AS INTEGER, ycoord AS INTEGER, letter AS STRING, col AS INTEGER, dn AS INTEGER)

  IF dn >= 0 THEN
    darray(dn).x = xcoord
    darray(dn).y = 1
    darray(dn).tm = INT(RND * 3) + 1
    darray(dn).t = darray(dn).tm
    darray(dn).m = DROP.MODE.DISPLAY
    darray(dn).ep = ycoord + INT(RND * (20 - ycoord)) + 3
  END IF

  scene(xcoord, ycoord).c = ASC(letter)
  scene(xcoord, ycoord).dc = col

END SUB

SUB do.timer

  STATIC t AS LONG

  t = 0
  WHILE (t < cps)
    t = t + 1
  WEND

END SUB

SUB DrawDrops

  DIM i AS INTEGER

  FOR i = 0 TO dnum - 1 STEP 1
    IF darray(i).m = DROP.MODE.CLEAR THEN
      IF scene(darray(i).x, darray(i).y).c = 0 THEN
        LOCATE darray(i).y, darray(i).x: PRINT " "
      END IF
    ELSE
      LOCATE darray(i).y, darray(i).x
      IF scene(darray(i).x, darray(i).y).c = 0 THEN
        COLOR 2, 0: PRINT CHR$(AcceptableChar)
      ELSE
        CreateTeap darray(i).x, darray(i).y
      END IF
    END IF
    darray(i).t = darray(i).t - 1
    IF darray(i).t <= 0 THEN
      darray(i).y = darray(i).y + 1
      IF darray(i).y > darray(i).ep THEN
        darray(i).m = INT(RND * 2) + 1
        darray(i).y = 1
        darray(i).x = INT(RND * 79) + 1
        IF (darray(i).m = DROP.MODE.CLEAR) THEN
          darray(i).tm = INT(RND * 2) + 1
          darray(i).ep = 23
        ELSE
          darray(i).tm = INT(RND * 5) + 1
          darray(i).ep = 15 + INT(RND * 8) + 1
        END IF
      END IF
      darray(i).t = darray(i).tm
    END IF
    IF darray(i).m = DROP.MODE.DISPLAY THEN
      LOCATE darray(i).y, darray(i).x: COLOR 10, 0: PRINT CHR$(AcceptableChar)
    END IF
  NEXT i

  i = 0
  WHILE i < ntep
    COLOR scene(tep(i).x, tep(i).y).dc, 0: LOCATE tep(i).y, tep(i).x
    IF tep(i).cd > 1 THEN
      PRINT CHR$(AcceptableChar)
      tep(i).cd = tep(i).cd - 1
    ELSE
      PRINT CHR$(scene(tep(i).x, tep(i).y).c)
    END IF
    i = i + 1
  WEND

END SUB

SUB ini.timer (fps AS LONG)
  DIM milli AS LONG, gettime AS LONG
 
  stoptime = 0
  milli = 1
  ON TIMER(TIMER.PRECISION) GOSUB Finish
  TIMER ON
  WHILE (stoptime = 0)
    milli = milli + 1
  WEND
  TIMER OFF
 
  cps = milli / TIMER.PRECISION / fps

END SUB

SUB Initialize

  DIM x AS INTEGER, y AS INTEGER, i AS INTEGER

  RANDOMIZE TIMER

  SCREEN 0
  COLOR 7, 0
  CLS

  lastcs = 0
  ntep = 0

  PRINT "Building acceptable characters (ach) table ..."

  ' 128 - 173 =  46 chars
  ' 224 - 254 = +31 chars
  '             ---
  '             77 total chars

  nach = 77

  FOR i = 128 TO 173 STEP 1
    ach(i - 128) = i
  NEXT i

  FOR i = 224 TO 254 STEP 1
    ach(i - 178) = i
  NEXT i
 
  PRINT "Creating drops ... "
 
  FOR i = 0 TO dnum - 1 STEP 1
    darray(i).x = 1
    darray(i).y = 1
    darray(i).tm = 4
    darray(i).t = darray(i).tm
    darray(i).ep = 23
    darray(i).m = DROP.MODE.CLEAR
  NEXT i
 
  PRINT "Clearing scene ..."

  FOR y = 1 TO 23 STEP 1
    FOR x = 1 TO 79 STEP 1
      scene(x, y).c = 0
      scene(x, y).active = 0
    NEXT x
  NEXT y

  PRINT "Defining title ... "

  DefScene 27, 9, "T", YELLOW, 11
  DefScene 29, 9, "H", YELLOW, 10
  DefScene 31, 9, "E", YELLOW, 9
  DefScene 35, 9, "Q", YELLOW, 8
  DefScene 37, 9, "B", YELLOW, 7
  DefScene 39, 9, "-", YELLOW, 6
  DefScene 41, 9, "M", YELLOW, 5
  DefScene 43, 9, "A", YELLOW, 4
  DefScene 45, 9, "T", YELLOW, 3
  DefScene 47, 9, "R", YELLOW, 2
  DefScene 49, 9, "I", YELLOW, 1
  DefScene 51, 9, "X", YELLOW, 0

  CreateTeap 71, 23
  CreateTeap 72, 23
  CreateTeap 73, 23
  CreateTeap 74, 23
  CreateTeap 75, 23
  CreateTeap 76, 23
  CreateTeap 77, 23
  CreateTeap 78, 23
  CreateTeap 79, 23
 
  DefScene 71, 23, "", CYAN, -1
  DefScene 72, 23, "c", CYAN, -1
  DefScene 73, 23, "e", CYAN, -1
  DefScene 74, 23, "G", CYAN, -1
  DefScene 75, 23, "o", CYAN, -1
  DefScene 76, 23, "b", CYAN, -1
  DefScene 77, 23, "l", CYAN, -1
  DefScene 78, 23, "i", CYAN, -1
  DefScene 79, 23, "n", CYAN, -1
 
  PRINT "Initializing timer ... "
  ini.timer (24)

  CLS

END SUB

