/****************************** File Prologue *******************************

University of Maryland University College
Introductory Programming in C/C++
CMIS-140
Section xxxx

~~~~~~~~~~
ASSIGNMENT
~~~~~~~~~~
Programming Project 6

~~~~~~~~~
FILE NAME
~~~~~~~~~
rational.cpp

~~~~~~
AUTHOR
~~~~~~
<Your Name>
<Your e-mail address>

~~~~~~~~~~~
DESCRIPTION
~~~~~~~~~~~

1. Analysis

    1.1 Rational class implements a rudimentary rational (fraction) data type.
    The data is maintained in the form of two integers representing the
    numerator and denominator of a fraction.

    1.2 Inputs (Definition, Type, Format, Valid Range)
    N/A

    1.3 Outputs (Definition, Type, Format, Valid Range)
    <N/A>

    1.4 Error Handling

    Divide by zero (i.e., a Rational number with zero denominator) will abort
    the client code (using the assert function).

    1.5 Special Considerations
    Client code must take special care NOT TO send a denominator value of
    zero when initializing a rational number.

2. Design

    The Rational class shall provide the following functions
    for the client code. See the description of each function for details.

       Rational Add
       Rational Subtract
       Rational Multiply
       Rational Divide
       bool     IsEqualTo
       void     Read
       void     Write
       long     Numerator
       long     Denominator;
       void     Set

****************************************************************************/

/*------------------------------ [INCLUDES] -------------------------------*/
#include <cassert>      // for using the assert() function
#include <cstdlib>      // for using labs() function
#include "rational.h"   // for using class Rational


/*----------------------------- [DATA TYPES] ------------------------------*/
//NONE


/*--------------------------- [GLOBAL CONSTANTS] --------------------------*/
//NONE


/*--------------------------- [GLOBAL VARIABLES] --------------------------*/
//NONE

/*-------------------------- [FUNCTION PROTOTYPES] ------------------------*/

// returns the greatest common divisor of num1 and num2
long gcd(long num1, long num2);

/*---------------------------- [FUNCTION BODIES] --------------------------*/

/////////////////////////////////////////////////////////////////////////////
// The following functions have been provided in their entirety:
//
// Rational (long, long);
// Read
// gcd
// Reduce
//
// The following functions have been provided as a stub (i.e., without the body)
// The student is required to complete these functions by producing the code
// to implement the described functionality.
//
// Rational ();
// Write
//
// The remaining member functions must be produced by the student.
/////////////////////////////////////////////////////////////////////////////

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Rational

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function is the default constructor for the rational class. It implicitly
initializes any Rational object that was not explicitly initialized by the
creator of the object to 0/1.


Formal Parameter    I/O                         Description
================    ===       ===============================================
none                 -        -

****************************************************************************/
// default constructor;  initializes fraction to 0/1
Rational::Rational()
: m_numerator   (0)
, m_denominator (1)
{
    // initialization of variables done above
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Rational

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function is a general constructor for Rational class. It initializes a
Rational object that was explicitly initialized by the creator of the object
with the values passed to its parameters.

The function reduces the newly created and initialized fraction to its
simplest form.

Precondition:  denominator != 0

Formal Parameter    I/O                         Description
================    ===       ===============================================
numerator           in        the value with which to initialize the numerator
denominator         in        the value with which to initialize the denominator

****************************************************************************/
Rational::Rational(long numerator, long denominator)
{
    assert(denominator != 0);

    m_numerator   = numerator;
    m_denominator = denominator;

    Reduce();
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Add

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

Adds this rational number to the specified fraction and returns the result
in a new value.

Formal Parameter    I/O                         Description
================    ===       ===============================================
otherFrac           in        fraction to add this value to

****************************************************************************/
Rational Rational::Add (Rational otherFrac) const
{
    long denominator = gcd(m_denominator, otherFrac.m_denominator);

    // addition is done by converting both fractions to have common
    // denominator and then adding the two. The new fraction inherits the
    // common denominator and is then reduced.

    // calculate common-based numerators for this and other based on gcd
    long thisValue = m_numerator * (denominator / m_denominator);
    long otherValue = otherFrac.m_numerator *
                        (denominator / otherFrac.m_denominator);

    // the new numerator is the addition of the common-based numerators
    Rational result(thisValue + otherValue, denominator);

    return result;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Subtract

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

Subtracts the specified fraction from this rational value and returns the
result in a new value.

Formal Parameter    I/O                         Description
================    ===       ===============================================
otherFrac           in        fraction to subtract from this value

****************************************************************************/
Rational Rational::Subtract (Rational otherFrac) const
{
    long denominator = gcd(m_denominator, otherFrac.m_denominator);

    // subtraction is done by converting both fractions to have common
    // denominator and then subtracting the second numerator from the
    // first.  The new fraction inherits the common denominator and is
    // then reduced.

    // calculate common-based numerators for this and other based on gcd
    long thisValue = m_numerator * (denominator / m_denominator);
    long otherValue = otherFrac.m_numerator *
                        (denominator / otherFrac.m_denominator);

    // the new numerator is the difference of the common-based numerators
    Rational result(thisValue - otherValue, denominator);

    return result;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Multiply

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

Multiplies this rational value by the specified fraction and returns the
product in a new value.

Formal Parameter    I/O                         Description
================    ===       ===============================================
otherFrac           in        fraction to multiply to this value

****************************************************************************/
Rational Rational::Multiply (Rational otherFrac) const
{
    Rational result;

    // multiplication of fractions is done simply by multiplying both
    // numerators together, both denominators, and then reducing the result
    result.Set(m_numerator * otherFrac.m_numerator,
                m_denominator * otherFrac.m_denominator);

    return result;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Add

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

Adds this rational number to the specified fraction and returns the result
in a new value.

Precondition:  otherFrac.m_numerator != 0 && otherFrac.m_denominator != 0

Formal Parameter    I/O                         Description
================    ===       ===============================================
otherFrac           in        fraction to add this value to

****************************************************************************/
Rational Rational::Divide (Rational otherFrac, DivideStatus& divStat) const
{
    Rational result;

    // there is no need to check the denominator because if the numerator is
    // zero then the result is a zero value regardless
    if (otherFrac.m_numerator == 0)
    {
        divStat = DIVIDE_BY_ZERO;

        // because the result has not been set to anything, this will return a
        // fraction with a zero value
    }
    else
    {
        // do division by multiplying this numerator to the other fraction's
        // denominator, multiplying this denominator by the other fraction's
        // numerator, and then reducing the result:
        //
        // n1   d2   n3
        // -- * -- = --
        // d1 * n2   d3

        result.Set(m_numerator * otherFrac.m_denominator,
                    m_denominator * otherFrac.m_numerator);

        divStat = SUCCESS;
    }

    return result;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
IsEqualTo

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

Determines if the current rational value is equal to the specified fraction.

Precondition:  otherFrac.m_numerator != 0 && otherFrac.m_denominator != 0

Formal Parameter    I/O                         Description
================    ===       ===============================================
otherFrac           in        fraction to compare this value to
<return value>      out       true if the values are equal, false otherwise

****************************************************************************/
bool Rational::IsEqualTo (Rational otherFrac) const
{
    long denominator = gcd(m_denominator, otherFrac.m_denominator);

    // equality is tested by first converting both fractions to the same
    // denominator and then comparing their numerators.

    // calculate common-based numerators for this and other based on gcd
    long thisValue = m_numerator * (denominator / m_denominator);
    long otherValue = otherFrac.m_numerator *
                        (denominator / otherFrac.m_denominator);

    bool result = true;

    if (thisValue != otherValue)
        result = false;

    return result;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Read

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function reads two integers from the input stream in a n/m format and
assigns it to the numerator and denominator of the Rational object that is
invoking the function.


Formal Parameter    I/O                         Description
================    ===       ===============================================
is                  inout     Input stream from which to read the numerator and
                              denominator

****************************************************************************/
void Rational::Read (istream& is)
{
    char c;
    int num, den;

    // read numerator and hopefully the '/'
    is >> num >> c;

    if (is && c != '/')
    {
        // if the next character is not a '/'

        is.putback(c);  // put the character back in the stream because
                        // it may belong to another part of the program

        // set the fail flag for the stream
        is.clear(ios::badbit | is.rdstate());
    }

    // handle denominator
    is >> den;

    // if denominator is zero, set the fail flag on the input stream
    if (den == 0)
        is.clear(ios::badbit | is.rdstate());

    // if input stream is still good, set the new values for the object
    // numerator and denominator, otherwise, don't change the object
    if (is)
    {
        m_numerator = num;
        m_denominator = den;
    }

}


/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Write

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function writes the Rational object that is invoking the function to
its output stream parameter.


Formal Parameter    I/O                         Description
================    ===       ===============================================
os                  inout     Output stream to write the value of the object

****************************************************************************/
void Rational::Write (ostream& os) const
{
    os << m_numerator << '/' << m_denominator;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Numerator

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function returns the value of the numerator in the rational value.

Formal Parameter    I/O                         Description
================    ===       ===============================================
<return value>      out       Rational object's numerator.

****************************************************************************/
long Rational::Numerator() const
{
    return m_numerator;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Numerator

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function returns the value of the denominator in the rational value.

Formal Parameter    I/O                         Description
================    ===       ===============================================
<return value>      out       Rational object's denominator.

****************************************************************************/
long Rational::Denominator() const
{
    return m_denominator;
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Set

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function sets the value of the rational object using the parameter
values for numerator and denominator.

Precondition:  denominator != 0

Formal Parameter    I/O                         Description
================    ===       ===============================================
numerator           in        Numerator to use.
denominator         in        Denominator to use (cannot be 0).

****************************************************************************/
void Rational::Set(long numerator, long denominator)
{
    assert (denominator != 0);

    m_numerator = numerator;
    m_denominator = denominator;

    Reduce();
}

/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
gcs

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function calculates the Greatest Common Divisor (GCD) of two integers.


Formal Parameter    I/O                         Description
================    ===       ===============================================
a, b                in        Two integers
<return value>      out       GCD of a and b

****************************************************************************/
long gcd(long a, long b)
{
    long a1 = labs(a);
    long b1 = labs(b);
    long temp;

    while (b1!= 0)
    {
        temp = a1;
        a1 = b1;
        b1 = temp % b1;
    }
    return a1;
}


/************************** Function Description ****************************

~~~~~~~~~~~~~
FUNCTION NAME
~~~~~~~~~~~~~
Reduce

~~~~~~~~~~~~~~~~~~~~
FUNCTION DESCRIPTION
~~~~~~~~~~~~~~~~~~~~

This function reduces the Rational object invoking it to its simplest form.

Formal Parameter    I/O                         Description
================    ===       ===============================================
none

****************************************************************************/
void Rational::Reduce()
{
    if (m_numerator == 0)
        m_denominator = 1;
    else
    {
        long divisor = gcd(m_numerator, m_denominator);

        m_numerator /= divisor;
        m_denominator /= divisor;

        if (m_denominator < 0)
        {
            m_denominator = -m_denominator;
            m_numerator = -m_numerator;
        }
    }
}