/*****************************************************************************

	[market.cpp] Supermarket Simulation
	By Julio Arias

DESCRIPTION:

	Write a program that simulates a checkout line at a supermarket.  The line is a queue object.  Customers (i.e., customer objects) arrive in random integer intervals of 1 to 4 minutes.  Also, each customer is served in random integer intervals of 1 to 4 minutes.  Obviously, the rates need to be balanced.  If the average arrival rate is larger than the average service rate, the queue will grow indefinetly.  Even with "balanced" rates, randomness can still cause long lines.  Run the supermarket simulation for a 12-hour day (720 minutes) using the following algorithm:
	
	1) Choose a random integer between 1 and 4 to determine the minute at which the first customer arrives.
	
	2) At the first customer's arrival time:
		Determine customer's service time (random integer from 1 to 4);
		Begin servicing the customer;
		Schedule arrival time of next customer (random integer 1 to 4 added to the current time).
	3) For each minute of the day:
		If the next customer arrives,
			Say so,
			Enqueue the customer;
			Schedule the arrival time of the next customer;
		If service was completed for the last customer;
			Say so
			Dequeue next customer to be serviced
			Determine customer's service completion time
				(random integer from 1 to 4 added to the current time).

	Now run your simulation for 720 minutes and answer each of the following:
		a) What is the maximum number of customers in the queue at any time?
			-> 5
		b) What is the longest wait any one customer experiences?
			-> 13 minutes
		c) What happens if the arrival integer is changed from 1-to-4 minutes to 1-to-3 minutes?
			-> The maximum customers in the queue at any one time (56) and the longest service time (115) are both increased by about 10 times.

*****************************************************************************/

#include <stdlib.h>				// rand(), srand()
#include <time.h>				// time()
#include "customer.h"			// Customer class
#include "queue.h"				// Queue class

using std::endl;
using std::cout;

//_[main()]___________________________________________________________________
//
//
int main()
{
	long	iMinute		= 0,	// current simulated minute
			iMaxInQueue	= 0,	// maximum number of users ever in queue
			iWait		= 0,	// wait of last serviced customer
			iMaxWait	= 0,	// longest wait of any one customer
			iNextArr	= 0,	// next customer's arrival time
			iService	= 0,	// time until current service is completed
			iArrTime,			// maximum arrival time of any one customer
			iSerTime,			// maximum service time of any one customer
			iSimTime;			// number of simulated minutes

	Customer		c;			// general use Customer object
	Queue<Customer> q;			// our queue of customers

	srand(time(NULL));

	//_[get input before starting]____________________________________________
	cout << "SuperMarket Simulation" << endl;

	cout << endl << "Longest interval between customers [4]? ";
	cin >> iArrTime;

	if (iArrTime < 1)
	{
		cout << "Defaulted to 4" << endl;
		iArrTime = 4;
	}

	cout << endl << "Maximum length of service time [4]? ";
	cin >> iSerTime;

	if (iSerTime < 1)
	{
		cout << "Defaulted to 4" << endl;
		iSerTime = 4;
	}

	cout << endl << "Number of Minutes to Simulate [720]? ";
	cin >> iSimTime;
	if (iSimTime < iArrTime)
	{
		cout << "Defaulted to 720" << endl;
		iSimTime = 720;
	}

	//_[process first customer]_______________________________________________
	iMinute = (rand()%iArrTime) + 1;
	cout << endl << "Minute " << iMinute << endl;
	c.reset(iMinute);
	q.enqueue(c);
	cout << "  First customer has arrived (" << c << ")" << endl;

	//_[run minute-based loop for simulation]_________________________________
	iMinute++;
	while (iMinute <= 720)
	{
		cout << "Minute " << iMinute << endl;

		//-[check to see if customer arrives]---------------------------------
		if (iMinute >= iNextArr)
		{
			c.reset(iMinute);
			q.enqueue(c);
			cout << "  New customer has arrived (" << c << ")" << endl;
			iNextArr = (rand()%iArrTime) + 1 + iMinute;
			if (q.isEmpty())
				iService = (rand()%iSerTime) + 1 + iMinute;
		}

		if ( !q.isEmpty() )
		{
			//-[check to see if service for customer is finished]-------------
			if (iMinute >= iService)
			{
				q.dequeue(c);
				iService = (rand()%iSerTime) + 1 + iMinute;
				iWait = iMinute - c.getArrival();
				cout << "  Service for " << c << " has been completed (waited " << iWait << " minutes)" << endl;

				if (iWait >= iMaxWait)
					iMaxWait = iWait;
			}
		}

		if (q.count() > iMaxInQueue)
			iMaxInQueue = q.count();
		iMinute++;
	}

	//_[output statistics]____________________________________________________
	cout << endl
		<< "Customers left in Queue after simulation: " << q.count() << endl
		<< "Maximum Customers in Queue at any time: " << iMaxInQueue << endl
		<< "Longest Wait Experienced: " << iMaxWait << endl;

	return 0;
}