//------------------------------------------------------------------------------
// File: triangle.h
// Desc: Declaration and Definitions for 'triangle' class.
//------------------------------------------------------------------------------
#ifndef _TRIANGLE_H
#define _TRIANGLE_H

//_[headers]____________________________________________________________________
//
#include "shape.h"			// shape base class
#include <iostream.h>		// i/o streaming (cout)

//_[triangle class]_____________________________________________________________
//
// Represents a RIGHT triangle which eases computations for area.  The first
// two member variables for side are assumed to be perpendicular and the third
// diagonal between the ends of those two:
//
//           |\
//           | \
// side 1 -> |  \ <- side 3
//           |___\
//
//             ^
//          side 2
//
class triangle : public shape
{
public:

	//--------------------------------------------------------------------------
	// Name: triangle()
	// Desc: Constructor.
	//--------------------------------------------------------------------------
	triangle(float NewSide1 = 3.0, float NewSide2 = 4.0, float NewSide3 = 5.0)
	: Side1	(NewSide1)
	, Side2	(NewSide2)
	, Side3	(NewSide3)
	{
	}

	//--------------------------------------------------------------------------
	// Name: print()
	// Desc: Displays info about triangle (see comment for 'shape::print()').
	//--------------------------------------------------------------------------
	virtual print()
	{
		cout << "Triangle:" << endl;
		cout << "Area of triangle is " << area() << endl;
		cout << "Perimeter of triangle is " << perimeter() << endl;
		cout << "Sides of triangle are " << Side1 << ", " << Side2 << ", "
			<< Side3 << endl;
	}

protected:

	//--------------------------------------------------------------------------
	// Name: area()
	// Desc: Calculates the area of the right triangle.  The first two sides
	//		are multiplied together and the result is divided by two.  This
	//		makes the assumption that the first two sides represent the
	//		perpendicular edges of the triangle so the third side is irrelevant
	//		to calculating the area.
	//--------------------------------------------------------------------------
	float area()
	{
		return (Side1 * Side2) / 2;
	}

	//--------------------------------------------------------------------------
	// Name: perimeter()
	// Desc: Calculates the perimeter of the triangle.
	//--------------------------------------------------------------------------
	float perimeter()
	{
		return Side1 + Side2 + Side3;
	}

	//--------------------------------------------------------------------------
	// Name: Side1-3
	// Desc: Lengths of each side.
	//--------------------------------------------------------------------------
	float Side1, Side2, Side3;

}; // triangle

#endif