#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include<stdlib.h>
#include<string.h>
#include<ctype.h>
// your header file gets included here, too!
#include"words.h"

// Function Definitions:

WordNode *LookUpWord( WordNode *first, char *word ) {
  // return position (address) of word in the list
  // return NULL if not there

  WordNode *ptr = first;

  while ( ptr ) {

    if (strcmp(ptr->word, word) == 0)
      break;

    ptr = ptr->next;

  }

  return ptr;
}

void ProcessWord( WordNode *&list, char *word, int location ) {
  // If word isn't in list, add it so the list remains alphabetically sorted
  // Otherwise increment the frequency

  // insure the word is lower-case
  char *lword = new char [strlen(word) + 1];
  char *pt = word, *wp = lword;
  while ( *pt ) {
    *wp = tolower(*pt);
    pt++;
    wp++;
  }
  *wp = '\0';

  // attempt to look up word in list
  WordNode *ptr = LookUpWord( list, lword );

  // if word was not found in list, add it
  if ( ptr == NULL ) {

    // check the word against the first in the list
    if ( list == NULL || strcmp(lword, list->word) < 0 ) {
      AddAtBeginning( list, lword, location );
    }
    // otherwise add it somewhere in the middle or at the end
    else {

      for ( ptr = list ; ptr->next ; ptr = ptr->next ) {
        if ( strcmp(lword, ptr->next->word) < 0) {
          AddInMiddle( ptr, lword, location );
          break;
        }
      }

      // if no place in middle was found, add at the end
      if ( ptr->next == NULL ) {
        AddAtEnd( list, lword, location );
      }

    }

  }
  // otherwise increment the frequency and add new location
  else {
    ptr->freq++;
    AddLoc( ptr, location );
  }

  // remove 'lword'
  delete[] lword;

  return;
}

void DisplayList( WordNode *list ) {
  // Displays list neatly

  // check for empty list
  if ( !list )
  {
    cout << "List is empty!" << endl;
    return;
  }

  // loop once through the entire list to determine the longest word
  unsigned int longest = strlen(list->word);
  WordNode *ptr = list->next;

  while ( ptr ) {

    if (strlen(ptr->word) > longest)
      longest = strlen(ptr->word);

    ptr = ptr->next;
  }

  // reset ptr and output each word
  LocNode *loc = NULL;
  ptr = list;

  while( ptr ) {

    // each word is in a fixed-length column so that the frequencies are aligned
    cout << setiosflags(ios::left) << setw(longest) << ptr->word << ':'
        << ptr->freq << " (";

    // print locations
    loc = ptr->loc;
    cout << loc->location;
    loc = loc->next;

    while ( loc ) {
      cout << ", " << loc->location;
      loc = loc->next;
    }

    // end line
    cout << ")" << endl;

    // move to next node
    ptr = ptr->next;
  }

  return;
}

void InitFile( ifstream &input ) {
  // gets a filename, and tries to open the file.
  // eg: input.open("test.data");

  // get the filename from user input ... getline() is used because the
  // filename is a character array with a limit that cannot be exceeded.
  char filename[256];
  cout << "Please enter file name: ";
  cin.getline(filename, 256);

  // open the file
  input.open( filename );

  // check for valid input
  while( !input ) {
    cout << "No such file exists!" << endl;
    cout << "Please enter file name: ";
    cin.getline(filename, 256);
    input.open( filename );
  }

  return;
}

WordNode *MakeNewNode( char *word, int location ) {
  // TWO calls to operator new are needed
  // one for the node, one for the word

  WordNode *newnode = new WordNode;
  newnode->word = new char [strlen(word) + 1];
  newnode->next = 0;
  newnode->freq = 1;
  newnode->loc = MakeNewLocNode(location);
  strcpy(newnode->word, word);

  return newnode;
}

void AddAtBeginning( WordNode *&first, char *word, int location ) {
  // Adds word to beginning of word list

  // create new word node
  WordNode *newnode = MakeNewNode( word, location );

  // set the new word node's next to the old list head
  newnode->next = first;

  // set the list head pointer to the new node
  first = newnode;

  return;
}

void AddAtEnd( WordNode *last, char *word, int location ) {
  // Adds word to end of word list

  // check for empty list
  if (last != NULL) {

    // make sure we're on the last node
    while ( last->next != NULL ) {
      last = last->next;
    }

    // create new node at end
    last->next = MakeNewNode( word, location );
  }

  return;
}

void AddInMiddle( WordNode *here, char *word, int location ) {
  // Adds word to middle of word list (based on 'here')

  // check for empty list
  if (here == NULL)  {
    AddAtBeginning( here, word, location );
  }
  // otherwise add at this spot (here)
  else {
    // create new word node
    WordNode *newnode = MakeNewNode( word, location );

    // set the new word node's next to the old next at 'here'
    newnode->next = here->next;

    // set the next pointer to our new node
    here->next = newnode;
  }

  return;

}

void RemoveFirst( WordNode *&node ) {
  // removes the first node ('node')

  // save next down from head to ptr
  WordNode *ptr = node->next;

  // remove location list
  RemoveLocList( node );

  // remove head
  delete[] node->word;
  delete node;

  // set first node to saved ptr
  node = ptr;
}

WordNode* Remove( WordNode *&list, char *word ) {
  // removes the node specified

  WordNode *Next = 0;

  // check for NULL
  if ( list == NULL ) {
    return Next;
  }

  // if first word in list, remove head
  if ( strcmp(list->word, word) == 0 ) {

    // remove list head
    RemoveFirst( list );

    // return new list head as next word
    Next = list;
  }
  // otherwise find word
  else {

    WordNode *ptr = list;

    // look to 'ptr->next' as word to remove
    while ( ptr->next && strcmp(ptr->next->word, word) ) {
      ptr = ptr->next;
    }

    // only remove if match was found in 'ptr->next' (otherwise not found)
    if ( ptr->next ) {

      // get node to be destroyed
      WordNode *victim = ptr->next;

      // drop from list
      Next = ptr->next = victim->next;

      // remove node
      RemoveLocList( victim );
      delete[] victim->word;
      delete victim;
    }
  }

  return Next;
}

void RemoveTrivials( WordNode *&list, int &count ) {
  // Removes all trivial words from a word list

  WordNode *ptr = list;

  // initialize count of trivials
  count = 0;

  // loop through list nodes
  while ( ptr ) {

    // remove word if trivial (length is <= 3 characters)
    if ( strlen(ptr->word) <= 3 ) {

      // increment count
      count++;

      // remove word and go to old position (removed word's next)
      ptr = Remove( list, ptr->word );

    }
    // otherwise move to next node
    else {
      ptr = ptr->next;
    }
  }

  return;
}

void DestroyList( WordNode *&list ) {
  // Frees all memory used by a list of word nodes

  while ( list ) {
    RemoveFirst( list );
  }

  return;
}

void AddLoc( WordNode *node, int location ) {
  // Adds a new location node to a list of locations for word node

  // check for NULL
  if ( node->loc == NULL )
    return;

  LocNode *loc = node->loc;

  // find last node in location list
  while ( loc->next != NULL ) {
    loc = loc->next;
  }

  // add node to next
  loc->next = MakeNewLocNode( location );

  return;
}

LocNode* MakeNewLocNode( int location ) {
  // Creates a new location node

  LocNode *newloc = new LocNode;
  newloc->location = location;
  newloc->next = NULL;

  return newloc;
}

void RemoveLocList( WordNode *node ) {
  // Removes a location list from a word node

  LocNode *loc = node->loc;

  while ( loc ) {
    // get next down from head
    loc = loc->next;

    // remove head
    delete node->loc;

    // set head to next down
    node->loc = loc;
  }

  return;
}
